@echo off
title Sacrifice Games Search Tool V3.1 by Stefan Pohl, www.sp-cc.de
@echo ***************************************************************
@echo ***     Sacrifice Games Search and Statistics Tool V3.1     ***
@echo ***           (C) 2022, Stefan Pohl, www.sp-cc.de           ***
@echo ***************************************************************
@echo ***     This tool uses pgn-extract (C) David J. Barnes      ***
@echo ***************************************************************
@echo ***************************************************************
@echo Enter the name of your games pgn-file (without .pgn ending!) The file must be in the working folder!
set /p filename=
set gamebase=%filename%.pgn
@echo *********************************************************************************
@echo enter 0 (or just Return-key) for all sacrifices (1-5+ pawnunits) and statistics
@echo enter 1,2,3,4 or 5 for searching 1,2,3,4 or 5+ pawnunits sacrifices (no statistics, but 2x faster). 
@echo Mention, that higher sacs are always included. Example: If 2 is chosen, all sacs from 2+ up to 5+
@echo sacs are searched and found...
@echo Special search option 9 for queen sacs, only. Mention, this means not 9+ pawnunits sacs, but 5+ 
@echo pawnunits sacs with a queen for the loosing color and no queen for winner...
set /p input=
set /A whatwork=input
if %whatwork% LSS 0 set /A whatwork=0
if %whatwork% GTR 5 set /A whatwork=9

@echo *********************************************************************************
@echo Enter the maximum length of all non-draw games, which the tool has to investigate. For human games
@echo or adjucated engine games, at least 80 moves are recommended, for non-adjucated engine games, at
@echo least 90 or 100 is recommended. Values below 40 will be set to 80 and above 250 will be set to 250
@echo off
set /p moves=
set /A movelimit=moves
if %movelimit% LSS 40 set /A movelimit=80
if %movelimit% GEQ 250 set /A movelimit=250
@echo *********************************************************************************

REM *************************************
REM *** Start of code, do not change! ***
REM *************************************
CALL :StartTimer
if %whatwork% EQU 0 goto fullsearch

del games_with_sacrifices.pgn 2>nul
fsutil file createnew games_with_sacrifices.pgn 0 >NUL 
del sacgames_1_pawns.pgn 2>nul
del sacgames_2_pawns.pgn 2>nul
del sacgames_3_pawns.pgn 2>nul
del sacgames_4_pawns.pgn 2>nul
del sacgames_5_pawns.pgn 2>nul
del sacgames_queensacs.pgn 2>nul

cd ./bin
del results.pgn 2>nul
del resultsd.pgn 2>nul
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
del blacksacs.pgn 2>nul
fsutil file createnew results.pgn 0 >NUL
fsutil file createnew resultsd.pgn 0 >NUL
fsutil file createnew whitewins.pgn 0 >NUL
fsutil file createnew blackwins.pgn 0 >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

@echo ******************************************************************************
@echo *** Filtering all non-draw games, movelimit=%movelimit%
@echo ******************************************************************************

REM *** filtering the white wins and black wins and put them in different files ***
pgn-extract --quiet --fixresulttags -bl15 -bu%movelimit% -Tr1-0 ../%gamebase% --output whitewins.pgn > NUL
pgn-extract --quiet --fixresulttags -bl15 -bu%movelimit% -Tr0-1 ../%gamebase% --output blackwins.pgn > NUL

if %whatwork% LEQ 5 (
@echo *** Searching for %whatwork%+ pawnunits sacrifices in the file containing white wins
) else (
@echo *** Searching for queen sacrifices in the file containing white wins
)
@echo ******************************************************************************
if %whatwork% EQU 1 pgn-extract --quiet -y1_pawnsac_white whitewins.pgn --output resultsd.pgn >NUL
if %whatwork% EQU 2 pgn-extract --quiet -y2_pawnsac_white whitewins.pgn --output resultsd.pgn >NUL
if %whatwork% EQU 3 pgn-extract --quiet -y3_pawnsac_white whitewins.pgn --output resultsd.pgn >NUL
if %whatwork% EQU 4 pgn-extract --quiet -y4_pawnsac_white whitewins.pgn --output resultsd.pgn >NUL
if %whatwork% EQU 5 pgn-extract --quiet -y5_pawnsac_white whitewins.pgn --output resultsd.pgn >NUL
if %whatwork% EQU 9 pgn-extract --quiet -yqueensac_white whitewins.pgn --output resultsd.pgn >NUL

if %whatwork% LEQ 5 (
@echo *** Searching for %whatwork%+ pawnunits sacrifices in the file containing black wins
) else (
@echo *** Searching for queen sacrifices in the file containing black wins
)
@echo ******************************************************************************
if %whatwork% EQU 1 pgn-extract --quiet -y1_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
if %whatwork% EQU 2 pgn-extract --quiet -y2_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
if %whatwork% EQU 3 pgn-extract --quiet -y3_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
if %whatwork% EQU 4 pgn-extract --quiet -y4_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
if %whatwork% EQU 5 pgn-extract --quiet -y5_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
if %whatwork% EQU 9 pgn-extract --quiet -yqueensac_black blackwins.pgn --output blacksacs.pgn >NUL

REM *** Merge all found games in one single file, then delete doubles ***
pgn-extract --quiet -aresultsd.pgn blacksacs.pgn >NUL
pgn-extract --quiet -D resultsd.pgn --output results.pgn >NUL

REM *** Count the number of games in the gamebase...
CALL :countgames results.pgn
set /A gesamtanzahl=%c_numb_pgn%

pgn-extract --quiet -o../games_with_sacrifices.pgn results.pgn >NUL

REM *** delete all working files ***
del results.pgn 2>nul
del resultsd.pgn 2>nul
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
del blacksacs.pgn 2>nul

cd ../

@echo ****************
@echo *** Finished ***
@echo ****************
if %gesamtanzahl% GTR 0 (
  @echo **************************************************************************
  @echo Number of games with sacrifices found in the database: %gesamtanzahl%
  @echo Mention, that the games are sorted by white wins, followed by black wins
  @echo All games with sacrifices stored in the file:
  @echo games_with_sacrifices.pgn
  @echo **************************************************************************
)
if %gesamtanzahl% EQU 0 (
  @echo **************************************************************************
  @echo No sacrifies with the chosen pawnunits found in the source-file, sorry.
  @echo **************************************************************************
  del games_with_sacrifices.pgn 2>nul
)
goto jumpover
:fullsearch
del games_with_sacrifices.pgn 2>nul

@echo *** Read pgn-file: %gamebase%
copy "%gamebase%" "bin/results.pgn" >NUL
cd ./bin

CALL :countgames results.pgn
set /A numb_origin=%c_numb_pgn%

if %numb_origin% LEQ 0 (
  @echo *************************************************************************
  @echo *** ERROR - no games found in the file 
  @echo *************************************************************************
  cd ../
  goto error
)

del results_opt1.pgn 2>nul
del results_opt2.pgn 2>nul 
del results_opt3.pgn 2>nul 
del results_opt4.pgn 2>nul 
del results_opt5.pgn 2>nul 
del results_opt9.pgn 2>nul 
del unique_opt1.pgn 2>nul 
del unique_opt2.pgn 2>nul 
del unique_opt3.pgn 2>nul 
del unique_opt4.pgn 2>nul 
del unique_opt5.pgn 2>nul 
del unique_opt9.pgn 2>nul 
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
del blacksacs.pgn 2>nul

fsutil file createnew results_opt1.pgn 0 >NUL
fsutil file createnew results_opt2.pgn 0 >NUL
fsutil file createnew results_opt3.pgn 0 >NUL
fsutil file createnew results_opt4.pgn 0 >NUL
fsutil file createnew results_opt5.pgn 0 >NUL
fsutil file createnew results_opt9.pgn 0 >NUL
fsutil file createnew whitewins.pgn 0 >NUL
fsutil file createnew blackwins.pgn 0 >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

@echo *** Number of games found in the source file: %numb_origin%
@echo *************************************************************************
@echo *** Filtering all non-draw games, movelimit=%movelimit%
@echo *************************************************************************
REM *** filtering the white wins and black wins and put them in different files ***
pgn-extract --quiet --fixresulttags -bl15 -bu%movelimit% -Tr1-0 results.pgn --output whitewins.pgn > NUL
pgn-extract --quiet --fixresulttags -bl15 -bu%movelimit% -Tr0-1 results.pgn --output blackwins.pgn > NUL

CALL :countgames whitewins.pgn
set /A whitewon=%c_numb_pgn%
CALL :countgames blackwins.pgn
set /A blackwon=%c_numb_pgn%
set /A numb_wins=%whitewon%+%blackwon%
@echo *** Number of won games found in the source file: %numb_wins%
@echo *************************************************************************

@echo *** Searching for 1+ pawnunits sacrifices in the file of white wins
@echo *************************************************************************
pgn-extract --quiet -y1_pawnsac_white whitewins.pgn --output results_opt1.pgn >NUL
del whitewins.pgn 2>nul
copy results_opt1.pgn whitewins.pgn > NUL
@echo *** Searching for 1+ pawnunits sacrifices in the file of black wins
@echo *************************************************************************
pgn-extract --quiet -y1_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt1.pgn blacksacs.pgn >NUL

del blackwins.pgn 2>nul
copy blacksacs.pgn blackwins.pgn > NUL

@echo *** Searching for 2-5+ pawnunits sacs
@echo *************************************************************************
pgn-extract --quiet -y2_pawnsac_white whitewins.pgn --output results_opt2.pgn >NUL
pgn-extract --quiet -y2_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt2.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt2.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

pgn-extract --quiet -y3_pawnsac_white whitewins.pgn --output results_opt3.pgn >NUL
pgn-extract --quiet -y3_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt3.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt3.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

pgn-extract --quiet -y4_pawnsac_white whitewins.pgn --output results_opt4.pgn >NUL
pgn-extract --quiet -y4_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt4.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt4.pgn blacksacs.pgn >NUL
fsutil file createnew blacksacs.pgn 0 >NUL

pgn-extract --quiet -y5_pawnsac_white whitewins.pgn --output results_opt5.pgn >NUL
pgn-extract --quiet -y5_pawnsac_black blackwins.pgn --output blacksacs.pgn >NUL
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
copy results_opt5.pgn whitewins.pgn > NUL
copy blacksacs.pgn blackwins.pgn > NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt5.pgn blacksacs.pgn >NUL

pgn-extract --quiet -yqueensac_white whitewins.pgn --output results_opt9.pgn >NUL
pgn-extract --quiet -yqueensac_black blackwins.pgn --output blacksacs.pgn >NUL
REM *** Merge all found games in one single file ***
pgn-extract --quiet -aresults_opt9.pgn blacksacs.pgn >NUL

REM *** Delete games in the gamebases with lower sacs, which already occur in higher sacs files...***
fsutil file createnew unique_opt1.pgn 0 >NUL
fsutil file createnew unique_opt2.pgn 0 >NUL
fsutil file createnew unique_opt3.pgn 0 >NUL
fsutil file createnew unique_opt4.pgn 0 >NUL
fsutil file createnew unique_opt5.pgn 0 >NUL
fsutil file createnew unique_opt9.pgn 0 >NUL

pgn-extract --quiet -D -ounique_opt9.pgn results_opt9.pgn >NUL
pgn-extract --quiet -cresults_opt9.pgn -D -ounique_opt5.pgn results_opt5.pgn > NUL
pgn-extract --quiet -cresults_opt5.pgn -D -ounique_opt4.pgn results_opt4.pgn > NUL
pgn-extract --quiet -cresults_opt4.pgn -D -ounique_opt3.pgn results_opt3.pgn > NUL
pgn-extract --quiet -cresults_opt3.pgn -D -ounique_opt2.pgn results_opt2.pgn > NUL
pgn-extract --quiet -cresults_opt2.pgn -D -ounique_opt1.pgn results_opt1.pgn > NUL

REM *** Count the number of games in the result bases...***
CALL :countgames unique_opt9.pgn
set /A numb_opt9=%c_numb_pgn%
CALL :countgames unique_opt5.pgn
set /A numb_opt5=%c_numb_pgn%
CALL :countgames unique_opt4.pgn
set /A numb_opt4=%c_numb_pgn%
CALL :countgames unique_opt3.pgn
set /A numb_opt3=%c_numb_pgn%
CALL :countgames unique_opt2.pgn
set /A numb_opt2=%c_numb_pgn%
CALL :countgames unique_opt1.pgn
set /A numb_opt1=%c_numb_pgn%

REM *** Count the number off all sacs found (add all 6 sac-numbers) ***
set /A numb_sum=%numb_opt1%+%numb_opt2%+%numb_opt3%+%numb_opt4%+%numb_opt5%+%numb_opt9%

cd ../
fsutil file createnew statistics.txt 0 >NUL

REM *** Build the statistics.txt file ***
@echo *************************************************************** >statistics.txt
@echo ***     Sacrifice Games Search and Statistics Tool V3.1     *** >>statistics.txt
@echo ***           (C) 2022, Stefan Pohl, www.sp-cc.de           *** >>statistics.txt
@echo *************************************************************** >>statistics.txt
@echo *** Sacrifice-statistics of the file %gamebase% >>statistics.txt
@echo *************************************************************** >>statistics.txt
@echo ************************************************************************************* >>statistics.txt
@echo Source pgn-file: %gamebase% >>statistics.txt
@echo ************************************************************************************* >>statistics.txt
@echo Number of games found in the source file: %numb_origin% >>statistics.txt
@echo ************************************************************************************* >>statistics.txt
@echo Number of won games found in the source file: %numb_wins% >>statistics.txt
@echo ************************************************************************************* >>statistics.txt
@echo Max. allowed length of games, investigated: %movelimit% moves >>statistics.txt
@echo ************************************************************************************* >>statistics.txt
CALL :percent %numb_origin% %numb_opt9%
set percent_overall=%percent%
CALL :percent %numb_wins% %numb_opt9%
set percent_of_wins=%percent%
@echo Number of Queen sacrifices (5+)  : %numb_opt9% (%percent_overall% of all games) (%percent_of_wins% of won games) >>statistics.txt

CALL :percent %numb_origin% %numb_opt5%
set percent_overall=%percent%
CALL :percent %numb_wins% %numb_opt5%
set percent_of_wins=%percent%
@echo Number of 5+ pawnunits sacrifices: %numb_opt5% (%percent_overall% of all games) (%percent_of_wins% of won games) >>statistics.txt

CALL :percent %numb_origin% %numb_opt4%
set percent_overall=%percent%
CALL :percent %numb_wins% %numb_opt4%
set percent_of_wins=%percent%
@echo Number of 4 pawnunits sacrifices : %numb_opt4% (%percent_overall% of all games) (%percent_of_wins% of won games) >>statistics.txt

CALL :percent %numb_origin% %numb_opt3%
set percent_overall=%percent%
CALL :percent %numb_wins% %numb_opt3%
set percent_of_wins=%percent%
@echo Number of 3 pawnunits sacrifices : %numb_opt3% (%percent_overall% of all games) (%percent_of_wins% of won games) >>statistics.txt

CALL :percent %numb_origin% %numb_opt2%
set percent_overall=%percent%
CALL :percent %numb_wins% %numb_opt2%
set percent_of_wins=%percent%
@echo Number of 2 pawnunits sacrifices : %numb_opt2% (%percent_overall% of all games) (%percent_of_wins% of won games) >>statistics.txt

CALL :percent %numb_origin% %numb_opt1%
set percent_overall=%percent%
CALL :percent %numb_wins% %numb_opt1%
set percent_of_wins=%percent%
@echo Number of 1 pawnunits sacrifices : %numb_opt1% (%percent_overall% of all games) (%percent_of_wins% of won games) >>statistics.txt

CALL :percent %numb_origin% %numb_sum%
set percent_overall=%percent%
CALL :percent %numb_wins% %numb_sum%
set percent_of_wins=%percent%
@echo Number of all sacrifices         : %numb_sum% (%percent_overall% of all games) (%percent_of_wins% of won games) >>statistics.txt
@echo ************************************************************************************* >>statistics.txt
@echo **************************************************** >>statistics.txt
@echo *** SGS-tool (C) 2022 Stefan Pohl (www.sp-cc.de) *** >>statistics.txt
@echo **************************************************** >>statistics.txt

del sacgames_1_pawns.pgn 2>nul
del sacgames_2_pawns.pgn 2>nul
del sacgames_3_pawns.pgn 2>nul
del sacgames_4_pawns.pgn 2>nul
del sacgames_5_pawns.pgn 2>nul
del sacgames_queensacs.pgn 2>nul

cd ./bin
REM *** Build final output pgn-files ***
pgn-extract --quiet -o../sacgames_1_pawns.pgn unique_opt1.pgn >NUL
pgn-extract --quiet -o../sacgames_2_pawns.pgn unique_opt2.pgn >NUL
pgn-extract --quiet -o../sacgames_3_pawns.pgn unique_opt3.pgn >NUL
pgn-extract --quiet -o../sacgames_4_pawns.pgn unique_opt4.pgn >NUL
pgn-extract --quiet -o../sacgames_5_pawns.pgn unique_opt5.pgn >NUL
pgn-extract --quiet -o../sacgames_queensacs.pgn unique_opt9.pgn >NUL


REM *** delete all working files ***
del results.pgn 2>nul
del results_opt1.pgn 2>nul
del results_opt2.pgn 2>nul 
del results_opt3.pgn 2>nul 
del results_opt4.pgn 2>nul 
del results_opt5.pgn 2>nul 
del results_opt9.pgn 2>nul 
del unique_opt1.pgn 2>nul 
del unique_opt2.pgn 2>nul 
del unique_opt3.pgn 2>nul 
del unique_opt4.pgn 2>nul 
del unique_opt5.pgn 2>nul 
del unique_opt9.pgn 2>nul 
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
del blacksacs.pgn 2>nul

cd ../
@echo ****************
@echo *** Finished ***
@echo ****************
@echo *** See the statistics.txt file for statistics
@echo *******************************************************
more statistics.txt
@echo **************************************************************************
@echo *** Games stored in 6 files: 
@echo *** sacgames_(1-5)_pawns.pgn and sacgames_queensacs.pgn
:jumpover
@echo **************************************************************************
CALL :StopTimer
CALL :DisplayTimerResult
@echo **************************************************************************
:error
CALL ./bin/playwav %windir%/media/alarm02.wav
pause

goto :EOF

:countgames
set /A c_numb_pgn=0
for /f "tokens=2,* delims= " %%G in ('find /C "[White " %~1') do set /A c_numb_pgn=%%H
exit /B 0

:percent
if %~1 LEQ 0 (
set percent=00.00%%
set /A percentx100=0
EXIT /B 0
)
set /A l_base=((1000000000/%~1)*%~2)
set /A l_percent=%l_base%/1000000
set /A l_rest1=%l_percent%%%10
set /A l_percent=%l_percent%/10
set /A l_secdig_pc=%l_base%/100000
set /A l_rest2=%l_secdig_pc%%%10
set /A l_thirddig_pc=%l_base%/10000
set /A l_rest3=%l_thirddig_pc%%%10

REM *** Round 2nd digit up, if 3rd digit is GEQ 5 and round 1st digit and percent-value up, if necessary
if %l_rest3% GEQ 5 set /A l_rest2+=1
if %l_rest2% GEQ 10 (
  set /A l_rest2-=10
  set /A l_rest1+=1
)
if %l_rest1% GEQ 10 (
  set /A l_rest1-=10
  set /A l_percent+=1
)

if %l_percent% LSS 10 (
  set percent=0%l_percent%.%l_rest1%%l_rest2%%%
) else (
  set percent=%l_percent%.%l_rest1%%l_rest2%%%
)
set /A percentx100=(%l_percent%*100)+(%l_rest1%*10)+%l_rest2%

if %l_percent% GTR 99 (
set percent=100.0%%
set /A percentx100=10000
)
EXIT /B 0

:StartTimer
set StartTIME=%TIME%
for /f "usebackq tokens=1-4 delims=:., " %%f in (`echo %StartTIME: =0%`) do set /a Start100S=1%%f*360000+1%%g*6000+1%%h*100+1%%i-36610100
EXIT /B 0

:StopTimer
set StopTIME=%TIME%
for /f "usebackq tokens=1-4 delims=:., " %%f in (`echo %StopTIME: =0%`) do set /a Stop100S=1%%f*360000+1%%g*6000+1%%h*100+1%%i-36610100
if %Stop100S% LSS %Start100S% set /a Stop100S+=8640000
set /a TookTime=%Stop100S%-%Start100S%
set TookTimePadded=0%TookTime%
EXIT /B 0

:DisplayTimerResult
set /A t_elapsed=%TookTime:~0,-2%.%TookTimePadded:~-2% 2>nul
set /A t_seconds=%t_elapsed%
set /A t_minutes=%t_seconds%/60
set /A t_hours=%t_minutes%/60
set /A t_seconds-=%t_minutes%*60
set /A t_minutes-=%t_hours%*60
if %t_seconds% LEQ 9 set t_seconds=0%t_seconds%
if %t_minutes% LEQ 9 set t_minutes=0%t_minutes%
if %t_hours% LEQ 9 set t_hours=0%t_hours%
@echo *** Elapsed time: %t_hours%:%t_minutes%:%t_seconds%
EXIT /B 0
